@@ -14,13 +14,13 @@ See the License for the specific language governing permissions and* limitations
 """
 
 import requests
-from . import CLIENT_VERSION
+from . import CerberusClientException, CLIENT_VERSION
 
-from .util import throw_if_bad_response
+from .util import throw_if_bad_response, get_with_retry, post_with_retry
 
 
 class UserAuth(object):
-    """Class to authenticate with username and password and returns vault token"""
+    """Class to authenticate with username and password and returns a cerberus token"""
     HEADERS = {"Content-Type": "application/json", "X-Cerberus-Client": "CerberusPythonClient/" + CLIENT_VERSION}
 
     def __init__(self, cerberus_url, username, password):
@@ -31,7 +31,7 @@ class UserAuth(object):
 
     def get_auth(self):
         """Returns auth response which has client token unless MFA is required"""
-        auth_resp = requests.get(self.cerberus_url + '/v2/auth/user',
+        auth_resp = get_with_retry(self.cerberus_url + '/v2/auth/user',
                                  auth=(self.username, self.password),
                                  headers=self.HEADERS)
 
@@ -54,12 +54,33 @@ class UserAuth(object):
 
     def get_mfa(self, auth_resp):
         """Gets MFA code from user and returns response which includes the client token"""
-        sec_code = input('Enter ' + auth_resp['data']['devices'][0]['name'] + ' security code: ')
+        devices = auth_resp['data']['devices']
+        if len(devices) == 1:
+            # If there's only one option, don't show selection prompt
+            selection = "0"
+            x = 1
+        else:
+            print("Found the following MFA devices")
+            x=0
+            for device in devices:
+                print("{0}: {1}".format(x, device['name']))
+                x = x + 1
 
-        mfa_resp = requests.post(
+            selection = input("Enter a selection: ")
+        if selection.isdigit():
+            selection_num=int(str(selection))
+        else:
+            raise CerberusClientException( str.join('', ["Selection: '", selection,"' is not a number"]))
+
+        if (selection_num >= x) or (selection_num < 0):
+            raise CerberusClientException(str.join('', ["Selection: '", str(selection_num), "' is out of range"]))
+
+        sec_code = input('Enter ' + auth_resp['data']['devices'][selection_num]['name'] + ' security code: ')
+
+        mfa_resp = post_with_retry(
             self.cerberus_url + '/v2/auth/mfa_check',
             json={'otp_token': sec_code,
-                  'device_id': auth_resp['data']['devices'][0]['id'],
+                  'device_id': auth_resp['data']['devices'][selection_num]['id'],
                   'state_token': auth_resp['data']['state_token']},
             headers=self.HEADERS
         )
